from datetime import datetime
from osgeo import gdal
from netcdf import netcdf as nc
import numpy as np
import os
import glob


def pixels_from_coordinates(lat, lon, max_y, max_x):
    """
    Return the 2 matrix with lat and lon of each pixel.

    Keyword arguments:
    lat -- A latitude matrix
    lon -- A longitude matrix
    max_y -- The max vertical pixels amount of an orthorectified image.
    max_x -- The max horizontal pixels amount of an orthorectified image.
    """
    # The degrees by pixel of the orthorectified image.
    x_ratio, y_ratio = max_x/360., max_y/180.
    x, y = np.zeros(lon.shape), np.zeros(lat.shape)
    x = (lon + 180.) * x_ratio
    y = (lat + 90.) * y_ratio
    return x, y


def transform_data(data, lat, lon):
    data = data if data.shape[0] > 1 else data[0]
    x, y = pixels_from_coordinates(lat, lon, data.shape[0], data.shape[1])
    result = np.zeros(x.shape, dtype=data.dtype)
    e_x = np.round(x)
    e_y = np.round(y)
    for (i,j), v in np.ndenumerate(e_x):
        tmp = data[int(e_y[i,j]),int(v)] # data[int(e_y[i,j]),int(e_x[i,j])]
        result[i,j] = tmp if tmp >= 0 else 0
    return result


localpath = os.path.dirname(__file__)


def obtain(date, compressed=True):
    """
    Return an integer matrix with the Linke turbidity multiplied by 20. So it
    should be divided by 20 in before the usage (over the CPU or the GPU).

    Keyword arguments:
    date -- the datetime of the file.
    compressed -- the true value if it should return a matrix of integers or
    if it should return an uncompressed matrix of floats.
    """
    pattern = "%s/tifs/%s_longlat_wgs84.tif"
    ds = gdal.Open(pattern % (localpath, str(date.month).zfill(2)))
    linke = ds.ReadAsArray()
    return linke if compressed else linke / 20.


def obtain_to(filename, compressed=True):
    """
    Return the linke turbidity projected to the lat lon matrix coordenates.

    Keyword arguments:
    filename -- the name of a netcdf file.
    """
    files = glob.glob(filename)
    root, _ = nc.open(filename)
    lat, lon = nc.getvar(root, 'lat')[0,:], nc.getvar(root, 'lon')[0,:]
    date_str = lambda f: ".".join((f.split('/')[-1]).split('.')[1:-2])
    date = lambda f: datetime.strptime(date_str(f), "%Y.%j.%H%M%S")
    linke = lambda f: transform_data(obtain(date(f), compressed), lat, lon)
    linkes_projected = map(lambda f: (f, linke(f)), files)
    nc.close(root)
    return dict(linkes_projected)


def persist(filename):
    """
    Append the linke turbidity projected (using lat lon) as variables of
    the netcdf file.

    Keyword arguments:
    filename -- the name of a netcdf file.
    """
    linkes_projected = obtain_to(filename, compressed=False)
    keys = sorted(linkes_projected.keys())
    values = np.vstack([map(lambda k: linkes_projected[k], keys)])
    with nc.loader(filename) as root:
        data = nc.getvar(root, 'data')
        linke = nc.getvar(root, 'linke', 'f4', source=data)
        linke[:] = values
