import unittest
from linketurbidity import instrument
import datetime
from netcdf import netcdf as nc
import numpy as np
import os
import itertools


class TestInstrument(unittest.TestCase):

    def initialize_file(self, filename):
        with nc.loader(filename) as root:
            nc.getdim(root, 'xc', 5)
            nc.getdim(root, 'yc', 3)
            nc.getdim(root, 'time')
            data = nc.getvar(root, 'data', 'i4', ('time', 'yc', 'xc'), fill_value=1.3)
            data[:] = 1.3
            nc.getvar(root, 'lat', 'f4', ('yc', 'xc'), fill_value=1.3)
            nc.getvar(root, 'lon', 'f4', ('yc', 'xc'), fill_value=1.3)

    def setUp(self):
        self.files = ['goes13.2014.124.123422.BAND_01.nc',
                      'goes13.2014.124.130422.BAND_01.nc']
        map(self.initialize_file, self.files)

    def tearDown(self):
        os.system('rm *.nc')

    def test_obtain(self):
        linke = instrument.obtain(datetime.datetime(2014, 11, 10))
        self.assertEquals(linke.shape, (2160, 4320))
        min_cond = linke[20 <= linke].all()
        max_cond = linke[linke <= 126].all()
        compressed_cond = linke.nbytes == 2160 * 4320 * 1
        self.assertTrue(min_cond & max_cond & compressed_cond)

    def test_obtain_uncompressed(self):
        linke = instrument.obtain(datetime.datetime(2014, 11, 10),
                                        compressed=False)
        self.assertEquals(linke.shape, (2160, 4320))
        min_cond = linke[1.0 <= linke].all()
        max_cond = linke[linke <= 7.0].all()
        uncompressed_cond = linke.nbytes == 2160 * 4320 * 8
        self.assertTrue(min_cond & max_cond & uncompressed_cond)

    def test_obtain_to(self):
        filename = self.files[0]
        linke = instrument.obtain_to(filename)[filename]
        self.assertEquals(linke.shape, (3, 5))
        min_cond = linke[20 <= linke].all()
        max_cond = linke[linke <= 126].all()
        uncompressed_cond = linke.nbytes == 3 * 5 * 1
        self.assertTrue(min_cond & max_cond & uncompressed_cond)
        values = np.matrix('71 71 71 71 71; 71 71 71 71 71; 71 71 71 71 71')
        self.assertTrue((linke == values).all())

    def test_obtain_to_uncompressed(self):
        filename = self.files[0]
        linke = instrument.obtain_to(filename,
                                          compressed=False)[filename]
        self.assertEquals(linke.shape, (3, 5))
        min_cond = linke[1.0 <= linke].all()
        max_cond = linke[linke <= 7.0].all()
        uncompressed_cond = linke.nbytes == 3 * 5 * 8
        self.assertTrue(min_cond & max_cond & uncompressed_cond)
        values = np.matrix('3.55 3.55 3.55 3.55 3.55;'
                           '3.55 3.55 3.55 3.55 3.55;'
                           '3.55 3.55 3.55 3.55 3.55')
        self.assertTrue((linke == values).all())

    def test_persist(self):
        instrument.persist(self.files[0])
        with nc.loader(self.files[0]) as root:
            linke = nc.getvar(root, 'linke')[:]
            values = np.matrix('3.55 3.55 3.55 3.55 3.55;'
                               '3.55 3.55 3.55 3.55 3.55;'
                               '3.55 3.55 3.55 3.55 3.55')
            test = lambda (x, y): self.assertAlmostEqual(linke[0, x, y],
                                                         values[x, y], 3)
            xs, ys = list(map(range, linke.shape[1:]))
            options = zip(xs * len(ys), ys * len(xs))
            map(test, options)

    def test_load_persist_using_netcdf_package(self):
        instrument.persist('goes13.2014.124.*.BAND_01.nc')
        with nc.loader(self.files) as root:
            linke = nc.getvar(root, 'linke')[:]
            values = np.matrix('3.55 3.55 3.55 3.55 3.55;'
                               '3.55 3.55 3.55 3.55 3.55;'
                               '3.55 3.55 3.55 3.55 3.55')
            values = np.vstack([[values], [values]])
            test = lambda (i, x, y): self.assertAlmostEqual(linke[i, x, y],
                                                         values[i, x, y], 3)
            i_s, x_s, y_s = list(map(range, linke.shape))
            options = zip(i_s * len(x_s) * len(y_s),
                          x_s * len(y_s) * len(i_s),
                          y_s * len(x_s) * len(i_s))
            map(test, options)


if __name__ == '__main__':
    unittest.main()
